#include "labelmap.h"
#include "wtruntime/klib/kbtree.h"
#include <stdlib.h>

struct label
{
	wt_u32      key;
	const char* value;
};

static int u32cmp(const wt_u32 lhs, const wt_u32 rhs)
{
	if(lhs<rhs) { return -1; }
	if(lhs>rhs) { return +1; }
	return 0;
}

#define label_cmp(a, b) (u32cmp((a).key, (b).key))
KBTREE_INIT(label, struct label, label_cmp)

struct labelmap* labelmap_create(size_t size)
{
	return (struct labelmap*)kb_init(label, size);
}

void labelmap_delete(struct labelmap* const map)
{
	kb_destroy(label, ((kbtree_t(label)*)map));
}

size_t labelmap_count(struct labelmap* const map)
{
	return ((kbtree_t(label)*)map)->n_keys;
}

const char* labelmap_find(struct labelmap* const map, const wt_u32 key)
{
	struct label elem;

	elem.key   = key;
	elem.value = NULL;

	{
		struct label* const res = kb_getp(label, ((kbtree_t(label)*)map), &elem);

		if(NULL!=res)
		{
			return res->value;
		}
	}

	return NULL;
}

int labelmap_add(struct labelmap* const map, const wt_u32 key, const char* const value)
{
	struct label elem;

	elem.key   = key;
	elem.value = value;

	{
		struct label* const res = kb_getp(label, ((kbtree_t(label)*)map), &elem);

		if(NULL!=res)
		{
			/* already exists */
			return -1;
		}
	}

	if(NULL==kb_putp(label, ((kbtree_t(label)*)map), &elem))
	{
		/* out of memory? */
		return -1;
	}

	return 0;
}

void labelmap_remove(struct labelmap* const map, const wt_u32 key)
{
	struct label elem;

	elem.key   = (wt_u32)key;
	elem.value = NULL;

	kb_delp(label, ((kbtree_t(label)*)map), &elem);
}
