/* $Id: ast_datatypecons.h 6310 2023-01-16 20:39:48Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef WTPARSER_AST_DATATYPECONS_H
#define WTPARSER_AST_DATATYPECONS_H

#include <string>
#include <deque>
#include "../astnode.h"
#include "../../wtruntime/wtruntime.h"

namespace wtast
{

class Type;
class DataType;

class DataTypeCons : public AstNode
{
	typedef std::deque<Type*> types_t;
	typedef std::deque<wt_u32> offsets_t;

	types_t     mTypes;
	offsets_t   mOffsets;
	std::string mName;
	wt_u32      mConsId;
	wt_u32      mDataSize;
	bool        mDefault;

public:
	DataTypeCons(const std::string & name)
		: AstNode()
		, mName(name)
		, mConsId(0)
		, mDataSize(sizeof(wt_u32)+sizeof(wt_u32)) // one for the ConsId, and one for the refcounter
		, mDefault(false)
	{
		// ok
	}

	~DataTypeCons();

	void Accept(AstVisitor* vis);

	DataTypeCons* Copy();

	Kind GetKind() const
	{
		return DATATYPECONS;
	}

	void AddTypeBack(Type* type);

	std::size_t GetNumberOfTypes() const
	{
		return mTypes.size();
	}

	Type* GetType(std::size_t idx)
	{
		if(idx<mTypes.size())
		{
			return mTypes[idx];
		}
		else
		{
			return 0;
		}
	}

	wt_u32 GetOffset(std::size_t idx)
	{
		if(idx<mOffsets.size())
		{
			return mOffsets[idx];
		}
		else
		{
			return 0;
		}
	}

	std::string & GetName()
	{
		return mName;
	}

	wt_u32 GetConsId() const
	{
		return mConsId;
	}

	bool IsDefault() const
	{
		return mDefault;
	}

	void SetDefault(bool def)
	{
		mDefault = def;
	}

	wt_u32 GetDataSize() const
	{
		return mDataSize;
	}

	bool HasFunctionType() const;

	bool HasFunctionType(const wtast::DataType* dt) const;

	bool IsCopyable() const;

	bool IsCopyable(const wtast::DataType* dt) const;

	friend class DataType;

private:

	void SetConsId(wt_u32 id)
	{
		mConsId = id;
	}
};

} // namespace wtast

#endif // WTPARSER_AST_DATATYPECONS_H
