/* $Id: ast_program.h 6310 2023-01-16 20:39:48Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef WTPARSER_AST_PROGRAM_H
#define WTPARSER_AST_PROGRAM_H

#include <string>
#include <deque>
#include <vector>
#include "../astnode.h"
#include "../astvisitor.h"
#include "../uniqueid.h"

namespace wtparser
{
class ParserHelper;
} // wtparser

namespace wtast
{

class Entity;

class Program : public AstNode
{
	typedef std::deque<Entity*> entities_t;
	entities_t mEntities;

	typedef std::vector<AstNode*> allocatednodes_t;
	allocatednodes_t mAllocatedAstNodes;

	wtparser::UniqueId mLastUniqueId;

	std::string mCompileDate;
	std::string mCompileTime;

public:
	Program()
		: AstNode()
	{
		// ok
	}

	template<typename Iter>
	Program(const Iter & first, const Iter & last)
		: AstNode()
		, mEntities(first, last)
	{
		// ok
	}

	~Program();

	void Accept(AstVisitor* vis);

	Program* Copy();

	Kind GetKind() const
	{
		return PROGRAM;
	}

	std::size_t GetNumberOfEntities() const
	{
		return mEntities.size();
	}

	Entity* GetEntity(std::size_t idx)
	{
		if(idx<mEntities.size())
		{
			return mEntities[idx];
		}
		else
		{
			return 0;
		}
	}

	wtparser::UniqueId GetLastUniqueId() const
	{
		return mLastUniqueId;
	}

	const std::string & GetCompileDate() const
	{
		return mCompileDate;
	}

	const std::string & GetCompileTime() const
	{
		return mCompileTime;
	}

	friend class wtparser::ParserHelper;
};

} // namespace wtast

#endif // WTPARSER_AST_PROGRAM_H
