//================================================================================================
// $Id: problemhandler.h 6309 2023-01-16 20:19:42Z aron $
//
// Project:
//		Welltype Parser
// Contents:
//		Problem handler
// Authors:
//		Aron Barath, 2014-2023
//================================================================================================
// Copyright (c) Aron Barath
//================================================================================================

#ifndef WTPARSER_PROBLEMHANDLER_H
#define WTPARSER_PROBLEMHANDLER_H

#include <string>
#include "frontend.h"

namespace wtparser
{

class ParserHelper;

class ProblemHandler
{
	ParserHelper*      mHelper;
	IProblemHandler*   mIProblemHandler;
	unsigned long long mProblemCount;

public:
	ProblemHandler(ParserHelper* helper, IProblemHandler* iph)
		: mHelper(helper)
		, mIProblemHandler(iph)
		, mProblemCount(0)
	{
		// ok
	}

	~ProblemHandler()
	{
		// ok
	}

	unsigned long long GetProblemCount() const
	{
		return mProblemCount;
	}

	void AddNote(const std::string & file, unsigned int line, const std::string & msg)
	{
		++mProblemCount;
		if(mIProblemHandler) { mIProblemHandler->AddNote(file, line, msg); }
	}

	void AddWarning(const std::string & file, unsigned int line, const std::string & msg)
	{
		++mProblemCount;
		if(mIProblemHandler) { mIProblemHandler->AddWarning(file, line, msg); }
	}

	void AddError(const std::string & file, unsigned int line, const std::string & msg)
	{
		++mProblemCount;
		if(mIProblemHandler) { mIProblemHandler->AddError(file, line, msg); }
	}
};

} // namespace wtparser

#endif // WTPARSER_PROBLEMHANDLER_H

